/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include "common/common.h"
#include "execution_defs.h"
#include "index/ix.h"
#include "system/sm.h"

class AbstractExecutor {
public:
    virtual ~AbstractExecutor() = default;

    virtual size_t tupleLen() const { return 0; };

    virtual const std::vector<ColMeta>& cols() const {
        std::vector<ColMeta>* _cols = nullptr;
        return *_cols;
    };

    virtual std::string getType() { return "AbstractExecutor"; };

    virtual void beginTuple() {};

    // TODO: Rename
    virtual void nextTuple() {};

    virtual std::unique_ptr<RmRecord> Next() = 0;

    virtual void Next(std::unique_ptr<RmRecord>& record) = 0;

    virtual bool is_end() const { return true; };

    virtual Rid& rid() = 0;

    std::vector<ColMeta>::const_iterator get_col(const std::vector<ColMeta>& rec_cols, const TabCol& target) {
        auto pos = std::find_if(rec_cols.begin(), rec_cols.end(), [&](const ColMeta& col) {
            return col.tab_name == target.tab_name && col.name == target.col_name &&
                (!col.aggr_op.has_value() || col.aggr_op.value() == target.aggr_op);
        });
        return pos;
    }

    size_t get_col_offset(const std::vector<ColMeta>& rec_cols, const TabCol& target) {
        auto pos = get_col(rec_cols, target);
        size_t size = pos - rec_cols.begin();
        size_t offset = 0;
        for (size_t i = 0; i < size; i++) {
            offset += rec_cols[i].len;
        }
        return offset;
    }

public:
    Rid abstract_rid_{};
    Context* context_{nullptr};
    std::vector<std::unique_ptr<AbstractExecutor>> children_{};
    bool group_end_{false};
};