/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once
#include "execution_defs.h"
#include "execution_manager.h"
#include "executor_abstract.h"
#include "executor_groupby.h"
#include "function/func.h"

class GroupbyExecutor;
class AggregateExecutor : public AbstractExecutor {
public:
    AggregateExecutor(std::unique_ptr<AbstractExecutor> prev, Context* context, std::vector<TabCol> cols, Aggregate_cache* cache_) {
        auto x = dynamic_cast<GroupbyExecutor*>(prev.get());
        empty_tuple_ = x == nullptr;

        children_.push_back(std::move(prev));
        context_ = context;
        cols_ = cols;
        len_ = 0;

        auto prev_metas_ = children_[0]->cols();
        for (auto& col : cols_) {
            auto pos = get_col(prev_metas_, col);
            col_metas_.push_back(*pos);
            col_metas_.back().offset = len_;
            col_metas_.back().aggr_op = col.aggr_op;
            if (col.aggr_op == AggrOp::AGGR_COUNT) {
                col_metas_.back().type = ColType::TYPE_INT;
                col_metas_.back().len = sizeof(int32_t);
            }
            len_ += col_metas_.back().len;
        }

        current_tuple_ = nullptr;

        cache = cache_;

        // for (auto& col : cols) {
        //     if (col.aggr_op == AggrOp::AGGR_COUNT) count_cols.push_back(col);
        //     else if (col.aggr_op == AggrOp::AGGR_MAX) max_cols.push_back(col);
        //     else if (col.aggr_op == AggrOp::AGGR_MIN) min_cols.push_back(col);
        //     else if (col.aggr_op == AggrOp::AGGR_SUM) sum_cols.push_back(col);
        // }
    }

    size_t tupleLen() const override { return len_; }

    const std::vector<ColMeta>& cols() const override { return col_metas_; }

    std::string getType() override { return "AggregateExecutor"; }

    //cache judge condition cache
    void beginTuple() override {
        int judge = Judge_cache();
        if (judge != -1 && is_cached) {
            if (cols_.size()==1) {
                if (cols_[0].aggr_op == AggrOp::AGGR_COUNT) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_count()[location]);
                    location = location + 1;
                    return;
                }
                else if (cols_[0].aggr_op == AggrOp::AGGR_MAX) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_max()[location]);
                    location = location + 1;
                    return;
                }
                else if (cols_[0].aggr_op == AggrOp::AGGR_MIN) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_min()[location]);
                    location = location + 1;
                    return;
                }
                else if (cols_[0].aggr_op == AggrOp::AGGR_SUM) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_sum()[location]);
                    location = location + 1;
                    return;
                }
            }
        }
        children_[0]->beginTuple();
        bool has_count = false;
        if (empty_tuple_) {
            has_count = InitTuple();
        }
        if (!doNext() && !has_count) {
            current_tuple_ = nullptr;
        }
    }

    void nextTuple() override {
        int judge = Judge_cache();
        if (judge != -1 && is_cached) {
            if (cols_.size()==1) {
                if (cols_[0].aggr_op == AggrOp::AGGR_COUNT) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_count()[location]);
                    location = location + 1;
                    return;
                }
                else if (cols_[0].aggr_op == AggrOp::AGGR_MAX) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_max()[location]);
                    location = location + 1;
                    return;
                }
                else if (cols_[0].aggr_op == AggrOp::AGGR_MIN) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_min()[location]);
                    location = location + 1;
                    return;
                }
                else if (cols_[0].aggr_op == AggrOp::AGGR_SUM) {
                    current_tuple_->MoveFrom(cache->get_cache()[judge].get_sum()[location]);
                    location = location + 1;
                    return;
                }
            }
        }
        if (!children_[0]->group_end_) {
            children_[0]->nextTuple();
        } else {
            children_[0]->group_end_ = false;
        }
        doNext();
    }

    std::unique_ptr<RmRecord> Next() override {
        auto ret = std::move(current_tuple_);

        if (cols_.size() ==1 ){
            if (cols_[0].aggr_op == AggrOp::AGGR_COUNT) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*current_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*sum_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*max_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*min_tuple_.get());
            }
            else if (cols_[0].aggr_op == AggrOp::AGGR_SUM) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*count_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*current_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*max_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*min_tuple_.get());
            }
            else if (cols_[0].aggr_op == AggrOp::AGGR_MAX) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*count_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*sum_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*current_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*min_tuple_.get());
            }
            else if (cols_[0].aggr_op == AggrOp::AGGR_MIN) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*count_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*sum_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*max_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*current_tuple_.get());
            }
        }
        
        current_tuple_ = nullptr;
        return ret;
    }

    // no chongfu suan
    void Next(std::unique_ptr<RmRecord>& record) override {
        record->MoveFrom(*current_tuple_.get());

        if (cols_.size() ==1 ){
            if (cols_[0].aggr_op == AggrOp::AGGR_COUNT) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*current_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*sum_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*max_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*min_tuple_.get());
            }
            else if (cols_[0].aggr_op == AggrOp::AGGR_SUM) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*count_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*current_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*max_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*min_tuple_.get());
            }
            else if (cols_[0].aggr_op == AggrOp::AGGR_MAX) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*count_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*sum_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*current_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*min_tuple_.get());
            }
            else if (cols_[0].aggr_op == AggrOp::AGGR_MIN) {
                cache->get_cache()[cache->get_location()].add_aggregation_count(*count_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_sum(*sum_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_max(*max_tuple_.get());
                cache->get_cache()[cache->get_location()].add_aggregation_min(*current_tuple_.get());
            }
        }

        current_tuple_ = nullptr;
    }

    bool is_end() const override { return !current_tuple_ && children_[0]->is_end(); }

    Rid& rid() override { return abstract_rid_; }

private:
    bool doNext() {
        auto& child = children_[0];
        bool jump_in = false;
        std::unique_ptr<RmRecord> tuple = std::make_unique<RmRecord>();
        std::unique_ptr<RmRecord> next_tuple = std::make_unique<RmRecord>();

        for (; !child->group_end_ && !child->is_end(); child->nextTuple()) {
            jump_in = true;
            if (!current_tuple_) {
                current_tuple_ = std::make_unique<RmRecord>(len_);

                count_tuple_ = std::make_unique<RmRecord>(len_);
                sum_tuple_ = std::make_unique<RmRecord>(len_);
                max_tuple_ = std::make_unique<RmRecord>(len_);
                min_tuple_ = std::make_unique<RmRecord>(len_);

                child->Next(tuple);

                size_t offset = 0;
                for (size_t i = 0; i < cols_.size(); ++i) {
                    auto& col_meta = col_metas_[i];
                    if (cols_[i].aggr_op == AggrOp::AGGR_COUNT) {
                        *(int32_t*)(current_tuple_->data + offset) = 1;
                        *(int32_t*)(count_tuple_->data+offset) = 1;
                    } else {
                        memcpy(current_tuple_->data + col_meta.offset, tuple->data + offset, col_meta.len);
                        memcpy(count_tuple_->data + col_meta.offset, tuple->data + offset, col_meta.len);
                    }
                    memcpy(sum_tuple_->data + col_meta.offset, tuple->data + offset, col_meta.len);
                    memcpy(max_tuple_->data + col_meta.offset, tuple->data + offset, col_meta.len);
                    memcpy(min_tuple_->data + col_meta.offset, tuple->data + offset, col_meta.len);
                    offset += col_meta.len;
                }
                continue;
            }

            child->Next(next_tuple);
            size_t offset = 0;
            for (size_t i = 0; i < cols_.size(); ++i) {
                auto& col = cols_[i];
                auto& col_meta = col_metas_[i];
                if (col.aggr_op == AggrOp::AGGR_NONE) {
                    offset += col_meta.len;
                    continue;
                }

                switch (col.aggr_op) {
                    case AggrOp::AGGR_COUNT: {
                        Count(col_meta.type, next_tuple->data + offset, current_tuple_->data + offset);

                        if (col_meta.type != ColType::TYPE_STRING) {
                            Sum(col_meta.type, next_tuple->data + offset, sum_tuple_->data + offset, col_meta.len);
                        }
                        Max(col_meta.type, next_tuple->data + offset, max_tuple_->data + offset, col_meta.len);
                        Min(col_meta.type, next_tuple->data + offset, min_tuple_->data + offset, col_meta.len);

                        break;
                    }
                    case AggrOp::AGGR_SUM: {
                        Sum(col_meta.type, next_tuple->data + offset, current_tuple_->data + offset, col_meta.len);

                        Count(col_meta.type, next_tuple->data + offset, count_tuple_->data + offset);
                        Max(col_meta.type, next_tuple->data + offset, max_tuple_->data + offset, col_meta.len);
                        Min(col_meta.type, next_tuple->data + offset, min_tuple_->data + offset, col_meta.len);

                        break;
                    }
                    case AggrOp::AGGR_MAX: {
                        Max(col_meta.type, next_tuple->data + offset, current_tuple_->data + offset, col_meta.len);

                        Count(col_meta.type, next_tuple->data + offset, count_tuple_->data + offset);
                        if (col_meta.type != ColType::TYPE_STRING) {
                            Sum(col_meta.type, next_tuple->data + offset, sum_tuple_->data + offset, col_meta.len);
                        }
                        Min(col_meta.type, next_tuple->data + offset, min_tuple_->data + offset, col_meta.len);

                        break;
                    }
                    case AggrOp::AGGR_MIN: {
                        Min(col_meta.type, next_tuple->data + offset, current_tuple_->data + offset, col_meta.len);

                        Count(col_meta.type, next_tuple->data + offset, count_tuple_->data + offset);
                        if (col_meta.type != ColType::TYPE_STRING) {
                            Sum(col_meta.type, next_tuple->data + offset, sum_tuple_->data + offset, col_meta.len);
                        }
                        Max(col_meta.type, next_tuple->data + offset, max_tuple_->data + offset, col_meta.len);

                        break;
                    }
                    default: {
                        break;
                    }
                }
                offset += col_meta.len;
            }
        }
        return jump_in;
    }

    bool InitTuple() {
        bool has_count = false;
        current_tuple_ = std::make_unique<RmRecord>(len_);

        count_tuple_ = std::make_unique<RmRecord>(len_);
        sum_tuple_ = std::make_unique<RmRecord>(len_);
        max_tuple_ = std::make_unique<RmRecord>(len_);
        min_tuple_ = std::make_unique<RmRecord>(len_);

        auto* data = current_tuple_->data;
        size_t offset = 0;

        for (size_t i = 0; i < cols_.size(); ++i) {
            auto& col_meta = col_metas_[i];
            size_t len = col_meta.len;
            switch (cols_[i].aggr_op) {
                case AggrOp::AGGR_COUNT: {
                    *(int32_t*)(data + offset) = 0;
                    has_count = true;

                    *(int32_t*)(sum_tuple_->data + offset) = 0;
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(max_tuple_->data + offset) = INT32_MIN;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(max_tuple_->data + offset) = std::numeric_limits<float>::lowest();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(max_tuple_->data + offset, 0, len);
                            break;
                        }
                    }
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(min_tuple_->data + offset) = INT32_MAX;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(min_tuple_->data + offset) = std::numeric_limits<float>::max();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(min_tuple_->data + offset, 0xff, len);
                            break;
                        }
                    }

                    break;
                }
                case AggrOp::AGGR_SUM: {
                    *(int32_t*)(data + offset) = 0;
                    has_count = true;

                    *(int32_t*)(count_tuple_->data + offset) = 0;
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(max_tuple_->data + offset) = INT32_MIN;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(max_tuple_->data + offset) = std::numeric_limits<float>::lowest();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(max_tuple_->data + offset, 0, len);
                            break;
                        }
                    }
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(min_tuple_->data + offset) = INT32_MAX;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(min_tuple_->data + offset) = std::numeric_limits<float>::max();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(min_tuple_->data + offset, 0xff, len);
                            break;
                        }
                    }

                    break;
                }
                case AggrOp::AGGR_MAX: {
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(data + offset) = INT32_MIN;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(data + offset) = std::numeric_limits<float>::lowest();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(data + offset, 0, len);
                            break;
                        }
                    }

                    *(int32_t*)(count_tuple_->data + offset) = 0;
                    *(int32_t*)(sum_tuple_->data + offset) = 0;
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(min_tuple_->data + offset) = INT32_MAX;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(min_tuple_->data + offset) = std::numeric_limits<float>::max();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(min_tuple_->data + offset, 0xff, len);
                            break;
                        }
                    }

                    break;
                }
                case AggrOp::AGGR_MIN: {
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(data + offset) = INT32_MAX;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(data + offset) = std::numeric_limits<float>::max();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(data + offset, 0xff, len);
                            break;
                        }
                    }

                    *(int32_t*)(count_tuple_->data + offset) = 0;
                    *(int32_t*)(sum_tuple_->data + offset) = 0;
                    switch (col_meta.type) {
                        case ColType::TYPE_INT: {
                            *(int32_t*)(max_tuple_->data + offset) = INT32_MAX;
                            break;
                        }
                        case ColType::TYPE_FLOAT: {
                            *(float*)(max_tuple_->data + offset) = std::numeric_limits<float>::max();
                            break;
                        }
                        case ColType::TYPE_STRING: {
                            memset(max_tuple_->data + offset, 0xff, len);
                            break;
                        }
                    }

                    break;
                }
                default: {
                    memset(data + offset, 0x00, len);
                }
            }

            offset += len;
        }
        return has_count;
    }

    // now, check aggregate attributes, where conditions and table name(suppose only one table)
    int Judge_cache() {
        int tail = cache->get_cache().size()-1;
        for (int i=0; i<cache->get_cache().size()-1; i++) {
            if (cache->get_cache()[i].is_valid == true) {
                if (cache->get_cache()[i].where_conds == cache->get_cache()[tail].where_conds) {
                    if (cache->get_cache()[i].max_col == cache->get_cache()[tail].max_col) {
                        if (cache->get_cache()[i].min_col == cache->get_cache()[tail].min_col) {
                            if (cache->get_cache()[i].sum_col == cache->get_cache()[tail].sum_col) {
                                if (cache->get_cache()[i].count_col == cache->get_cache()[tail].sum_col) {
                                    if (cache->get_cache()[i].table_list[0] == cols_[0].tab_name) {
                                        return i;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return -1;
    }

private:
    std::vector<TabCol> cols_;
    std::vector<ColMeta> col_metas_;
    size_t len_;
    std::unique_ptr<RmRecord> current_tuple_;
    bool empty_tuple_;

    //aggregation cache
    Aggregate_cache* cache;
    bool is_cached = false;
    int location = 0;
    // std::vector<TabCol> sum_cols;
    // std::vector<TabCol> max_cols;
    // std::vector<TabCol> min_cols;
    // std::vector<TabCol> count_cols;
    std::unique_ptr<RmRecord> count_tuple_;
    std::unique_ptr<RmRecord> max_tuple_;
    std::unique_ptr<RmRecord> min_tuple_;
    std::unique_ptr<RmRecord> sum_tuple_;
};