import table
from enum import Enum

class DeltaType(Enum):
    INSERT = 1
    DELETE = 2
    UPDATE = 3


class Delta:
    def __init__(self, t, name, columns, values=None):
        self.type = t
        self.name = name
        self.columns = columns
        self.values = values


class Database:
    def __init__(self, name, tables):
        self.name = name
        self.tables = tables
        self.delta = []
        self.new_orders = []
        self.temp_orders = []

    def add_delta(self, delta):
        self.delta.append(delta)

    def apply_delta(self):
        for d in self.delta:
            for table in self.tables:
                if table.name == d.name:
                    if d.type == DeltaType.INSERT:
                        table.insert(d.values)
                    elif d.type == DeltaType.DELETE:
                        table.delete(d.columns)
                    elif d.type == DeltaType.UPDATE:
                        table.update(d.values, d.columns)
        self.delta = []

benchmark_db = Database(
    "benchmark",
    [
        table.warehouse,
        table.district,
        table.item,
        table.stock,
        table.customer,
        table.history,
        table.orders,
        table.new_orders,
        table.order_line,
    ],
)
