class Column:
    def __init__(self, name, type, len=None):
        self.name = name
        self.type = type
        self.len = len


class Table:
    def __init__(self, name, columns):
        self.name = name
        self.columns = columns
        self.data = []
        self.value_map = {}

    def insert(self, values):
        self.data.append(values)
        for i, v in enumerate(values):
            if i not in self.value_map:
                self.value_map[i] = set()
            self.value_map[i].add(v)

    # only equal condition
    def update(self, values, conds):
        for row in self.data:
            ok = True
            for cond in conds:
                if row[cond[0]] != cond[1]:
                    ok = False
                    break
            if ok:
                for v in values:
                    if v[0] not in self.value_map:
                        print(v)
                        for j, v in enumerate(self.value_map):
                            print(j)
                    if v[0] in self.value_map and row[v[0]] in self.value_map[v[0]]:
                        self.value_map[v[0]].remove(row[v[0]])
                        self.value_map[v[0]].add(v[1])
                    row[v[0]] = v[1]

    def delete(self, conds):
        for row in self.data:
            ok = True
            for cond in conds:
                if row[cond[0]] != cond[1]:
                    ok = False
                    break
            if ok:
                for i, v in enumerate(row):
                    if i not in self.value_map:
                        print("row {}".format(i))
                        for j, v in enumerate(self.value_map):
                            print("value map: {}".format(j))
                    if i in self.value_map and v in self.value_map[i]:
                        self.value_map[i].remove(v)
                self.data.remove(row)

    def check(self, value):
        for i, v in enumerate(value):
            if v in self.value_map[i]:
                return False
        return True

    def ranges(self, col):
        return sorted(list(self.value_map[col]))


warehouse = Table(
    "warehouse",
    [
        Column("w_id", int),
        Column("w_name", str, 10),
        Column("w_street_1", str, 20),
        Column("w_street_2", str, 20),
        Column("w_city", str, 20),
        Column("w_state", str, 2),
        Column("w_zip", str, 9),
        Column("w_tax", float),
        Column("w_ytd", float),
    ],
)

district = Table(
    "district",
    [
        Column("d_id", int),
        Column("d_w_id", int),
        Column("d_name", str, 10),
        Column("d_street_1", str, 20),
        Column("d_street_2", str, 20),
        Column("d_city", str, 20),
        Column("d_state", str, 2),
        Column("d_zip", str, 9),
        Column("d_tax", float),
        Column("d_ytd", float),
        Column("d_next_o_id", int),
    ],
)

item = Table(
    "item",
    [
        Column("i_id", int),
        Column("i_im_id", int),
        Column("i_name", str, 24),
        Column("i_price", float),
        Column("i_data", str, 50),
    ],
)

stock = Table(
    "stock",
    [
        Column("s_i_id", int),
        Column("s_w_id", int),
        Column("s_quantity", int),
        Column("s_dist_01", str, 24),
        Column("s_dist_02", str, 24),
        Column("s_dist_03", str, 24),
        Column("s_dist_04", str, 24),
        Column("s_dist_05", str, 24),
        Column("s_dist_06", str, 24),
        Column("s_dist_07", str, 24),
        Column("s_dist_08", str, 24),
        Column("s_dist_09", str, 24),
        Column("s_dist_10", str, 24),
        Column("s_ytd", float),
        Column("s_order_cnt", int),
        Column("s_remote_cnt", int),
        Column("s_data", str, 50),
    ],
)

customer = Table(
    "customer",
    [
        Column("c_id", int),
        Column("c_d_id", int),
        Column("c_w_id", int),
        Column("c_first", str, 16),
        Column("c_middle", str, 2),
        Column("c_last", str, 16),
        Column("c_street_1", str, 20),
        Column("c_street_2", str, 20),
        Column("c_city", str, 20),
        Column("c_state", str, 2),
        Column("c_zip", str, 9),
        Column("c_phone", str, 16),
        Column("c_since", str, 30),
        Column("c_credit", str, 2),
        Column("c_credit_lim", int),
        Column("c_discount", float),
        Column("c_balance", float),
        Column("c_ytd_payment", float),
        Column("c_payment_cnt", int),
        Column("c_delivery_cnt", int),
        Column("c_data", str, 50),
    ],
)

history = Table(
    "history",
    [
        Column("h_c_id", int),
        Column("h_c_d_id", int),
        Column("h_c_w_id", int),
        Column("h_d_id", int),
        Column("h_w_id", int),
        Column("h_date", str, 19),
        Column("h_amount", float),
        Column("h_data", str, 24),
    ],
)

new_orders = Table(
    "new_orders",
    [
        Column("no_o_id", int),
        Column("no_d_id", int),
        Column("no_w_id", int),
    ],
)

orders = Table(
    "orders",
    [
        Column("o_id", int),
        Column("o_d_id", int),
        Column("o_w_id", int),
        Column("o_c_id", int),
        Column("o_entry_d", str, 19),
        Column("o_carrier_id", int),
        Column("o_ol_cnt", int),
        Column("o_all_local", int),
    ],
)

order_line = Table(
    "order_line",
    [
        Column("ol_o_id", int),
        Column("ol_d_id", int),
        Column("ol_w_id", int),
        Column("ol_number", int),
        Column("ol_i_id", int),
        Column("ol_supply_w_id", int),
        Column("ol_delivery_d", str, 30),
        Column("ol_quantity", int),
        Column("ol_amount", float),
        Column("ol_dist_info", str, 24),
    ],
)
