import os
import socket

def connect_to_database(server_host, server_port):
    try:
        sockfd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sockfd.connect((server_host, server_port))
        return sockfd
    except socket.error as e:
        print(f"Failed to connect: {e}")
        return None

def execute_sql_command(sockfd, sql_command):
    try:
        sockfd.sendall(sql_command.encode())
        recv_data = sockfd.recv(4096).decode()
        return recv_data
    except (socket.error, IOError) as e:
        print(f"Error communicating with server: {e}")

def execute_sql_file(sockfd, sql_file):
    try:
        with open(sql_file, 'r') as file:
            sql_commands = file.readlines()
        results = []
        for sql_command in sql_commands:
            if sql_command.strip():
                result = execute_sql_command(sockfd, sql_command)
    except IOError as e:
        print(f"Error reading SQL file: {e}")

def main():
    server_host = "127.0.0.1"
    server_port = 8765
    sql_dir = "../sql/Q7"  # SQL 文件目录
    expect_output_dir = "../expect_output/Q7"  # 预期输出文件目录
    output_file = "/home/yxr/db2024/build/test_runner/output.txt"  # 输出文件

    # 偏移量变量
    offset = 0

    sockfd = connect_to_database(server_host, server_port)
    if not sockfd:
        print("Failed to connect to the database server.")
        return

    try:
        # 扫描指定目录下的所有 SQL 文件
        
        for filename in sorted(os.listdir(sql_dir)):
            if filename.endswith(".sql"):
                sql_file = os.path.join(sql_dir, filename)
                expected_output_file = os.path.join(expect_output_dir, f"{os.path.splitext(filename)[0]}.txt")

                execute_sql_file(sockfd, sql_file)

                # 比较内存中的结果和预期输出文件，并记录偏移量
                offset = compare_and_update_offset(output_file, expected_output_file, offset, filename)
                print(offset)
    finally:
        sockfd.close()

def compare_and_update_offset(output_file, expected_output_file, offset, filename):
    try:
        with open(output_file, 'r') as f:
            f.seek(offset)
            results_lines = f.readlines()

        with open(expected_output_file, 'r') as f:
            expected_lines = f.readlines()

        # Find lines that are different
        diff_found = False
        for line_num, (results_line, expected_line) in enumerate(zip(results_lines, expected_lines), start=1):
            if results_line.strip() != expected_line.strip():
                diff_found = True
                # print(f"Difference found in file {filename} at line {line_num}:")
                # print(f"    Output: {results_line.strip()}")
                # print(f"    Expected: {expected_line.strip()}")

        if not diff_found:
            print(f"{filename}: PASS")
            offset += sum(len(line) for line in expected_lines)
        else:
            offset += sum(len(line) for line in expected_lines)
            print(f"{filename}: FAILED!")

        return offset

    except IOError as e:
        print(f"IOError: {e}")
        return offset

if __name__ == "__main__":
    main()
